//
//  SynthesizeSingleton.h
//
//  Created by Matt Gallagher on 20/10/08.
//  Copyright 2008 Matt Gallagher. All rights reserved.
//
//  Permission is given to use this source code file, free of charge, in any
//  project, commercial or otherwise, entirely at your risk, with the condition
//  that any redistribution (in part or whole) of source code must retain
//  this copyright and permission notice. Attribution in compiled projects is
//  appreciated but not required.
//

#define SynthesizeSingletonForClassWithAccessor(classname, accessorMethodName) \
 \
static classname *accessorMethodName = nil; \
 \
+ (classname *)accessorMethodName \
{ \
	@synchronized(self) \
	{ \
		if (accessorMethodName == nil) \
		{ \
			accessorMethodName = [[self alloc] init]; \
		} \
	} \
	 \
	return accessorMethodName; \
} \
 \
+ (id)allocWithZone:(NSZone *)zone \
{ \
	@synchronized(self) \
	{ \
		if (accessorMethodName == nil) \
		{ \
			accessorMethodName = [super allocWithZone:zone]; \
			return accessorMethodName; \
		} \
	} \
	 \
	return nil; \
} \
 \
- (id)copyWithZone:(NSZone *)zone \
{ \
	return self; \
} \
 \
- (id)retain \
{ \
	return self; \
} \
 \
- (NSUInteger)retainCount \
{ \
	return NSUIntegerMax; \
} \
 \
- (void)release \
{ \
} \
 \
- (id)autorelease \
{ \
	return self; \
}

#define SynthesizeSingletonForClass(classname) SynthesizeSingletonForClassWithAccessor(classname, shared##classname)
